diff a/src/hotspot/share/gc/shared/gcConfiguration.cpp b/src/hotspot/share/gc/shared/gcConfiguration.cpp
--- a/src/hotspot/share/gc/shared/gcConfiguration.cpp
+++ b/src/hotspot/share/gc/shared/gcConfiguration.cpp
@@ -30,32 +30,37 @@
 #include "memory/universe.hpp"
 #include "oops/compressedOops.hpp"
 #include "runtime/globals.hpp"
 #include "runtime/globals_extension.hpp"
 #include "utilities/debug.hpp"
+#include "utilities/macros.hpp"
 
 GCName GCConfiguration::young_collector() const {
   if (UseG1GC) {
     return G1New;
   }
 
   if (UseParallelGC) {
     return ParallelScavenge;
   }
 
+  if (UseShenandoahGC) {
+#if INCLUDE_SHENANDOAHGC
+    if (strcmp(ShenandoahGCMode, "generational") == 0) {
+      return Shenandoah;
+    }
+#endif
+    return NA;
+  }
+
   if (UseZGC) {
     if (ZGenerational) {
       return ZMinor;
     } else {
       return NA;
     }
   }
-
-  if (UseShenandoahGC) {
-    return NA;
-  }
-
   return DefNew;
 }
 
 GCName GCConfiguration::old_collector() const {
   if (UseG1GC) {
@@ -64,22 +69,26 @@
 
   if (UseParallelGC) {
     return ParallelOld;
   }
 
+  if (UseShenandoahGC) {
+#if INCLUDE_SHENANDOAHGC
+    if (strcmp(ShenandoahGCMode, "generational") == 0) {
+      return Shenandoah;
+    }
+#endif
+    return NA;
+  }
+
   if (UseZGC) {
     if (ZGenerational) {
       return ZMajor;
     } else {
       return Z;
     }
   }
-
-  if (UseShenandoahGC) {
-    return Shenandoah;
-  }
-
   return SerialOld;
 }
 
 uint GCConfiguration::num_parallel_gc_threads() const {
   return ParallelGCThreads;
